# standard
from pathlib import Path
# package
from .base import _BaseName

__all__ = ['Name', 'File', 'Pipe', 'PipeFile']


class Name(_BaseName):
    """Inherited by: :class:`naming.File` :class:`naming.Pipe`

    Base class for name objects.

    Each subclass may have its own `config` attribute that should be a dictionary in the form of {field: pattern}
    where pattern is a valid regular expression.

    Classes may as well have a `drops` iterable attribute representing the fileds they want to ignore from their bases
    and a `compounds` dictionary attribute for nesting existing fields into multiple new ones.

    All fields should be unique. No duplicates are allowed.

    ======  ==========
    **Config:**
    ------------------
    *base*  Accepts any amount of word characters
    ======  ==========

    Basic use::

        >>> from naming import Name
        >>> n = Name()
        >>> n.get_name()  # no name has been set on the object, convention is solved with [missing] fields
        '[base]'
        >>> n.values
        {}
        >>> n.set_name('hello_world')
        >>> n.values
        {'base': 'hello_world'}
        >>> n.base = 'fields_as_properties'
        >>> n.values
        {'base': 'fields_as_properties'}
    """
    config = dict(base=r'\w+')
    drops = tuple()
    compounds = dict()

    def __init__(self, name: str='', separator: str='_'):
        """Sets the patterns defined by the `config` attribute. If any extra work is to be done by the class init
        it should be implemented on the `_init_name_core` method.

        :param name: Name to initialize the object with. Defaults to an empty string and it can later be set
                     by calling the :func:`~naming.Name.set_name` method.
        :param separator: Separator for the name fields. Defaults to an underscore.
        """
        self.__keys = self.config.keys()
        self.__items = self.config.items()
        super().__init__(name, separator)

    def _set_values(self):
        super()._set_values()
        for k, v in self.__items:
            setattr(self, rf'_{k}', v)
        for ck, cv in self.compounds.items():
            setattr(self, rf'_{ck}', ''.join(getattr(self, rf'_{v}') for v in cv))

    def _set_patterns(self):
        super()._set_patterns()
        self._set_pattern(*self.__keys)

    def get_pattern_list(self):
        result = super().get_pattern_list()
        result.extend(rf'_{k}' for k in self.__keys if not (k in self.drops or k in self._compounds_fields))
        return result


class File(Name):
    """Inherited by: :class:`naming.PipeFile`

    File Name objects.

    ===========  ===========
    **Unique Fields:**
    ------------------------
    *extension*  Any amount of word characters
    ===========  ===========

    Basic use::

        >>> from naming import File
        >>> f = File()
        >>> f.get_name()
        '[basse].[extension]'
        >>> f.get_name(extension='png')
        '[base].png'
        >>> f.set_name('hello.world')
        >>> f.values
        {'base': 'hello', 'extension': 'world'}
        >>> f.extension
        'world'
        >>> f.extension = 'abc'
        >>> f.name
        'hello.abc'
        >>> f.values
        {'base': 'hello', 'extension': 'abc'}
    """
    def __init__(self, *args, **kwargs):
        self._cwd = kwargs.pop('cwd', None)
        super().__init__(*args, **kwargs)

    def _set_values(self):
        super()._set_values()
        self._extension = '[.](?P<extension>\w+)'
        self._add_field_property('extension')

    def _get_joined_pattern(self) -> str:
        return rf'{super()._get_joined_pattern()}{self._extension}'

    def get_name(self, **values) -> str:
        if not values and self.name:
            return super().get_name(**values)
        extension = values.get('extension') or self.extension or "[extension]"
        return rf'{super().get_name(**values)}.{extension}'

    def get_path_pattern_list(self):
        return []

    @property
    def cwd(self):
        return self._cwd

    @cwd.setter
    def cwd(self, value):
        self._cwd = value

    @property
    def path(self) -> Path:
        """The Path representing this object on the filesystem."""
        args = list(self._iter_translated_pattern_list('get_path_pattern_list'))
        args.append(self.get_name())
        return Path(*args)

    @property
    def full_path(self) -> Path:
        """The resolved full Path representing this object on the filesystem."""
        cwd = Path.home() if not self.cwd else Path(self.cwd)
        return Path.joinpath(cwd, self.path)


class Pipe(Name):
    """Inherited by: :class:`naming.PipeFile`

    Pipe Name objects.

    =========  =========
    **Unique Fields:**
    --------------------
    *output**  Any amount of word characters
    *version*  Any amount of digits
    *frame***  Any amount of digits
    \* optional field. ** exists only when *output* is there as well.
    ====================

    ======  ============
    **Composed Fields:**
    --------------------
    *pipe*  Combination of unique fields in the form of: (.{output})\*.{version}.{frame}**
    \* optional field. ** exists only when *output* is there as well.
    ====================

    Basic use::

        >>> from naming import Pipe
        >>> p = Pipe()
        >>> p.get_name()
        '[base].[pipe]'
        >>> p.get_name(version=10)
        '[base].10'
        >>> p.get_name(output='data')
        '[base].data.[version]'
        >>> p.get_name(output='cache', version=7, frame=24)
        '[base].cache.7.24'
        >>> p = Pipe('my_wip_data.1')
        >>> p.version
        '1'
        >>> p.values
        {'base': 'my_wip_data', 'version': '1'}
        >>> p.get_name(output='exchange')  # returns a new string
        'my_wip_data.exchange.1'
        >>> p.name
        'my_wip_data.1'
        >>> p.output = 'exchange'  # mutates the object
        >>> p.name
        'my_wip_data.exchange.1'
        >>> p.frame = 101
        >>> p.version = 7
        >>> p.name
        'my_wip_data.exchange.7.101'
        >>> p.values
        {'base': 'my_wip_data', 'output': 'exchange', 'version': '7', 'frame': '101'}
    """
    _pipe_fields = ('output', 'version', 'frame')

    def _set_values(self):
        super()._set_values()
        self._version = '\d+'
        self._output = '\w+'
        self._frame = '\d+'
        self._pipe = rf'(({self._pipe_separator_pattern}{self._output})?[.]{self._version}([.]{self._frame})?)'

    def _set_patterns(self):
        super()._set_patterns()
        self._set_pattern('pipe', *self._pipe_fields)

    def _get_joined_pattern(self):
        return rf'{super()._get_joined_pattern()}{self._pipe}'

    @property
    def pipe_separator(self) -> str:
        return '.'

    @property
    def _pipe_separator_pattern(self):
        return rf'\{self.pipe_separator}'

    @property
    def pipe_name(self) -> str:
        """The pipe name string of this object."""
        pipe_suffix = self.pipe or rf"{self.pipe_separator}[pipe]"
        return rf'{self.nice_name}{pipe_suffix}'

    def _filter_k(self, k):
        return k == 'pipe'

    def _format_pipe_field(self, k, v):
        if k == 'frame' and v is None:
            return ''
        return rf'{self.pipe_separator}{v if v is not None else rf"[{k}]"}'

    def _get_pipe_field(self, output=None, version=None, frame=None) -> str:
        fields = dict(output=output or None, version=version, frame=frame)
        # comparisons to None due to 0 being a valid value
        fields = {k: v if v is not None else getattr(self, k) for k, v in fields.items()}

        if all(v is None for v in fields.values()):
            suffix = rf'{self.pipe_separator}[pipe]'
            return self.pipe or suffix if self.name else suffix

        if not fields['output'] and fields['frame'] is None:  # optional fields
            return rf'.{fields["version"] or 0}'

        return ''.join(self._format_pipe_field(k, v) for k, v in fields.items())

    def get_name(self, **values) -> str:
        if not values and self.name:
            return super().get_name(**values)
        try:
            # allow for getting name without pipe field in subclasses e.g. .File
            pipe = values['pipe'] or ''
        except KeyError:
            kwargs = {k: values.get(k) for k in self._pipe_fields}
            pipe = self._get_pipe_field(**kwargs)
        return rf'{super().get_name(**values)}{pipe}'


class PipeFile(File, Pipe):
    """
    Basic use::

        >>> from naming import PipeFile
        >>> p = PipeFile('wipfile.7.ext')
        >>> p.values
        {'base': 'wipfile', 'version': '7', 'extension': 'ext'}
        >>> [p.get_name(frame=x, output='render') for x in range(10)]
        ['wipfile.render.7.0.ext',
        'wipfile.render.7.1.ext',
        'wipfile.render.7.2.ext',
        'wipfile.render.7.3.ext',
        'wipfile.render.7.4.ext',
        'wipfile.render.7.5.ext',
        'wipfile.render.7.6.ext',
        'wipfile.render.7.7.ext',
        'wipfile.render.7.8.ext',
        'wipfile.render.7.9.ext']
        >>> class ProjectFile(PipeFile):
        ...     config = dict(year='[0-9]{4}',
        ...                   user='[a-z]+',
        ...                   another='(constant)',
        ...                   last='[a-zA-Z0-9]+')
        ...
        >>> pf = ProjectFile('project_data_name_2017_christianl_constant_iamlast.data.17.abc')
        >>> pf.values
        {'base': 'project_data_name',
        'year': '2017',
        'user': 'christianl',
        'another': 'constant',
        'last': 'iamlast',
        'output': 'data',
        'version': '17',
        'extension': 'abc'}
        >>> pf.nice_name  # no pipe & extension fields
        'project_data_name_2017_christianl_constant_iamlast'
        >>> pf.year
        '2017'
        >>> pf.year = 'nondigits'  # mutating with invalid fields raises a NameError
        Traceback (most recent call last):
        ...
        NameError: Can't set invalid name 'project_data_name_nondigits_christianl_constant_iamlast.data.17.abc' on ProjectFile instance. Valid convention is: [base]_[year]_[user]_[another]_[last].[pipe].[extension]
        >>> pf.year = 1907
        >>> pf.name
        'project_data_name_1907_christianl_constant_iamlast.data.17.abc'
        >>> pf.extension
        'abc'
        >>> pf.separator = '  '  # you can set the separator to a different set of characters
        >>> pf.name
        'project_data_name  1907  christianl  constant  iamlast.data.17.abc'
    """
    pass



# standard
import re
import abc
import typing


def _regex_property(field_name: str) -> property:
    def getter(self):
        pattern = getattr(self, rf'__{field_name}')
        return rf'(?P<{field_name}>{pattern})'

    def setter(self, value):
        setattr(self, rf'__{field_name}', value)
    return property(getter, setter)


def _field_property(field_name: str) -> property:
    def getter(self):
        return self._values.get(field_name)

    def setter(self, value):
        new_name = self.get_name(**{field_name: value})
        self.set_name(new_name)
    return property(getter, setter)


class _ABCName(abc.ABCMeta):
    def __new__(mcs, name, bases, args):
        new_cls = super().__new__(mcs, name, bases, args)
        mcs._merge_bases_dict('config', new_cls, bases)
        mcs._merge_bases_dict('compounds', new_cls, bases)
        new_cls._compounds_fields = set().union(*[v for v in new_cls.compounds.values()])
        mcs._merge_bases_drops(new_cls, bases)
        return new_cls

    @staticmethod
    def _merge_bases_dict(name, new_cls, bases):
        base_maps = [getattr(b, name, {}) for b in bases]
        base_maps.append(getattr(new_cls, name, {}))
        new_map = {}
        for dic in base_maps:
            new_map.update(dic)
        setattr(new_cls, name, new_map)

    @staticmethod
    def _merge_bases_drops(new_cls, bases):
        new_drops = set(getattr(new_cls, 'drops', tuple()))
        for b in bases:
            for drop in getattr(b, 'drops', tuple()):
                new_drops.add(drop)
                new_cls.config.pop(drop, None)
        new_cls.drops = new_drops


class _BaseName(object, metaclass=_ABCName):
    """This is the base abstract class for Name objects. You should not need to subclass directly from this object.
    All subclasses are encouraged to inherit from Name instead of this one."""

    _regex_property_name = r'[a-zA-Z][\w]+'

    def __init__(self, name: str='', separator: str='_'):
        super().__init__()
        self.__values = {}
        self.__items = self.__values.items()
        self._set_separator(separator)
        self._set_patterns()
        self._init_name_core(name)

    def _init_name_core(self, name: str):
        """Runs whenever a Name object is initialized or its name is set."""
        self.__set_name(name)
        self._set_values()
        self.__set_regex()
        self.__validate()

    def _set_separator(self, separator: str):
        self._separator = rf'{separator}'
        self._separator_pattern = rf'\{separator}'

    @property
    def separator(self) -> str:
        """The string that acts as a separator of all the fields in the name."""
        return self._separator

    @separator.setter
    def separator(self, value: str):
        self._set_separator(value)
        name = self.get_name(**self.values) if self.name else None
        self._init_name_core(name)

    @abc.abstractmethod
    def _set_patterns(self):
        pass

    def _set_pattern(self, *patterns):
        for p in patterns:
            setattr(self.__class__, rf'_{p}', _regex_property(p))
            self._add_field_property(p)

    def _add_field_property(self, field_name):
        setattr(self.__class__, field_name, _field_property(field_name))

    def __set_name(self, name: str):
        self.__name = rf'{name}' if name else None

    @property
    def name(self) -> str:
        """The name string set on the object."""
        return self.__name

    @abc.abstractmethod
    def _set_values(self):
        pass

    def __set_regex(self):
        self.__regex = re.compile(r'^{}$'.format(self._get_joined_pattern()))

    def __validate(self):
        if not self.name:
            return
        self.set_name(self.name)

    def set_name(self, name: str):
        """Set this object's name to the provided string.

        :param name: The name to be set on this object.
        :raises NameError: If an invalid string is provided.
        """
        match = self.__regex.match(name)
        if not match:
            msg = ' '.join([rf"Can't set invalid name '{name}' on {self.__class__.__name__} instance.",
                            rf"Valid convention is: {self.__class__().get_name()}"])
            raise NameError(msg)
        self.__set_name(name)
        self.__values.update(match.groupdict())

    @property
    def _values(self) -> typing.Dict[str, str]:
        return self.__values

    @abc.abstractmethod
    def get_pattern_list(self) -> typing.List[str]:
        return []

    def _get_values_pattern(self) -> typing.List[str]:
        return [getattr(self, p) for p in self.get_pattern_list()]

    def _get_joined_pattern(self) -> str:
        return self._separator_pattern.join(self._get_values_pattern())

    @property
    def values(self) -> dict:
        """The field values of this object's name as a dictionary in the form of {field: value}."""
        return {k: v for k, v in self.__items if not self._filter_kv(k, v)}

    def _filter_kv(self, k: str, v) -> bool:
        if self._filter_k(k) or self._filter_v(v):
            return True

    def _filter_k(self, k: str):
        pass

    def _filter_v(self, v):
        return v is None

    @property
    def nice_name(self) -> str:
        """This object's pure name attribute."""
        return self._get_nice_name()

    def _get_nice_name(self, **values) -> str:
        return self._separator.join(self._iter_translated_pattern_list('get_pattern_list', **values))

    def get_name(self, **values) -> str:
        """Get a new name string from this object's name values.

        :param values: Variable keyword arguments where the **key** should refer to a field on this object that will
                       use the provided value to build the new name.
        """
        if not values and self.name:
            return self.name
        if values:
            for ck, cv in getattr(self, 'compounds', {}).items():
                if ck not in values:
                    compounds = []
                    for c in cv:
                        value = values.get(c) or getattr(self, c)
                        if value is None:  # 0 is a valid value
                            break
                        compounds.append(rf'{value}')
                    else:
                        values[ck] = ''.join(compounds)
        return self._get_nice_name(**values)

    def _iter_translated_pattern_list(self, pattern: str, **values) -> typing.Generator:
        self_values = self._values
        for p in getattr(self, pattern)():
            nice_name = re.search(self._regex_property_name, p).group(0)
            if nice_name in values:
                value = str(values[nice_name])
            elif self_values:
                try:
                    value = str(self_values[nice_name])
                except KeyError:
                    value = getattr(self, p)  # must be a valid property
            else:
                value = rf'[{nice_name}]'
            yield value

    def __str__(self):
        return self.get_name()

    def __repr__(self):
        name = self.name or ''
        return rf'{self.__class__.__name__}("{name}")'



# -*- coding: utf-8 -*-
"""
Names testing module.
"""
# standard
import os
import unittest
from pathlib import Path
# package
from naming import *


class TestName(unittest.TestCase):

    def test_empty_name(self):
        n = Name()
        self.assertEqual('[base]', n.get_name())
        self.assertEqual('[base]', str(n))

    def test_init_name(self):
        n = Name('initname')
        self.assertEqual('initname', n.get_name())
        with self.assertRaises(NameError):
            Name(dict(my_name='dict'))
        n = Name(dict())
        self.assertFalse(None, n.get_name())

    def test_set_name(self):
        n = Name()
        n.set_name('setname')
        self.assertEqual('setname', n.get_name())


class TestEasyName(unittest.TestCase):

    def test_empty_name(self):
        n = Name()
        self.assertEqual('[base]', n.get_name())
        self.assertEqual({}, n.values)

    def test_new_empty_name(self):
        extra_fields = dict(year='[0-9]{4}', username='[a-z]+', anotherfield='(constant)', lastfield='[a-zA-Z0-9]+')
        Project = type('Project', (Name,), dict(config=extra_fields))
        p = Project()
        self.assertEqual('[base]_[year]_[username]_[anotherfield]_[lastfield]', p.get_name())
        p.set_name('this_is_my_base_name_2017_christianl_constant_iamlast')
        self.assertEqual('2017', p.year)

    def test_set_name(self):
        extra_fields = dict(year='[0-9]{4}', username='[a-z]+', anotherfield='(constant)', lastfield='[a-zA-Z0-9]+')
        ProjectFile = type('ProjectFile', (PipeFile,), dict(config=extra_fields))
        pf = ProjectFile('this_is_my_base_name_2017_christianl_constant_iamlast.base.17.abc')
        self.assertEqual('this_is_my_base_name_2017_christianl_constant_iamlast', pf.nice_name)
        self.assertEqual('this_is_my_base_name_2017_christianl_constant_iamlast.base.17', pf.pipe_name)
        self.assertEqual('2017', pf.year)
        self.assertEqual('iamlast', pf.lastfield)
        self.assertEqual('abc', pf.extension)

    def test_separator(self):
        extra_fields = dict(year='[0-9]{4}', username='[a-z]+', anotherfield='(constant)', lastfield='[a-zA-Z0-9]+')
        Project = type('Project', (Name,), dict(config=extra_fields))
        p = Project('this_is_my_base_name_2017_christianl_constant_iamlast')
        self.assertEqual('_', p.separator)
        p.separator = '  '
        self.assertEqual('this_is_my_base_name  2017  christianl  constant  iamlast', p.name)


class TestPipe(unittest.TestCase):

    def test_empty_name(self):
        p = Pipe()
        self.assertEqual('[base].[pipe]', p.get_name())
        self.assertEqual('[base].10', p.get_name(version=10))
        self.assertEqual('[base].geo.10', p.get_name(version=10, output='geo'))
        self.assertEqual('[base].geo.10.25', p.get_name(version=10, output='geo', frame=25))
        self.assertEqual('[base].[output].10.25', p.get_name(version=10, frame=25))
        self.assertEqual('[base].[output].[version].101', p.get_name(frame=101))
        self.assertEqual('[base].cache.[version]', p.get_name(output='cache'))
        self.assertEqual('[base]', p.get_name(pipe=None))

    def test_empty_name_separator(self):
        p = Pipe()
        for sep in ' ', '.', '/', '/ .':
            p.separator = sep
            self.assertEqual(f'[base].[pipe]', p.get_name())
            self.assertEqual('[base].10', p.get_name(version=10))
            self.assertEqual(f'[base].geo.10', p.get_name(version=10, output='geo'))
            self.assertEqual(f'[base].geo.10.25', p.get_name(version=10, output='geo', frame=25))
            self.assertEqual(f'[base].[output].10.25', p.get_name(version=10, frame=25))
            self.assertEqual(f'[base].[output].[version].101', p.get_name(frame=101))
            self.assertEqual(f'[base].cache.[version]', p.get_name(output='cache'))
            self.assertEqual('[base]', p.get_name(pipe=None))

    def test_init_name(self):
        p = Pipe('initname.pipeline.0')
        self.assertEqual('initname.pipeline.0', p.get_name())
        self.assertEqual('.pipeline.0', p.pipe)
        self.assertEqual('pipeline', p.output)
        p = Pipe('initname.7')
        self.assertEqual('7', p.version)
        p = Pipe('initname.geo.0.1')
        self.assertEqual('0', p.version)
        self.assertEqual('1', p.frame)
        self.assertEqual('initname', p.nice_name)

        p = Pipe('name_with_underscores.pipeline.0')
        self.assertEqual('name_with_underscores.pipeline.0', p.get_name())
        self.assertEqual('.pipeline.0', p.pipe)
        self.assertEqual('pipeline', p.output)
        p = Pipe('name_with_underscores.7')
        self.assertEqual('7', p.version)
        p = Pipe('name_with_underscores.geo.0.1')
        self.assertEqual('0', p.version)
        self.assertEqual('1', p.frame)
        self.assertEqual('name_with_underscores', p.nice_name)

    def test_set_name(self):
        p = Pipe()
        p.set_name('setname.pipeline.0')
        self.assertEqual('.pipeline.0', p.pipe)
        self.assertEqual('pipeline', p.output)
        self.assertEqual('0', p.version)
        p.set_name('setname.pipeline.0.5')
        self.assertEqual('5', p.frame)
        self.assertEqual('setname', p.nice_name)

    def test_values(self):
        p = Pipe()
        p.set_name('my_pipe_file.1')
        self.assertEqual({'base': 'my_pipe_file', 'version': '1'}, p.values)

    def test_get_empty_name(self):
        p = Pipe()
        self.assertEqual('[base].[pipe]', p.pipe_name)
        self.assertEqual('[base].[pipe]', p.get_name())
        self.assertEqual('[base].out.7', p.get_name(pipe='.out.7'))
        self.assertEqual('[base].out.[version]', p.get_name(output='out'))
        self.assertEqual('[base].7', p.get_name(version=7))
        self.assertEqual('[base].[output].[version].101', p.get_name(frame=101))

    def test_get_init_name(self):
        p = Pipe('my_pipe_file.7')
        self.assertEqual('my_pipe_file.[output].7.101', p.get_name(frame=101))
        self.assertEqual('my_pipe_file.cache.7', p.get_name(output='cache'))
        self.assertEqual('my_pipe_file.8', p.get_name(version=int(p.version) + 1))


class TestFile(unittest.TestCase):

    def test_empty_name(self):
        f = File()
        self.assertEqual('[base].[extension]', f.get_name())
        self.assertEqual('[base].abc', f.get_name(extension='abc'))
        self.assertEqual('[base].[extension]', f.get_name(extension=''))

        f.set_name('myfile.ext')
        self.assertEqual('ext', f.extension)
        self.assertEqual('myfile', f.base)
        self.assertEqual(f.get_name(), str(f.path))
        self.assertEqual(os.path.join(Path.home(), f.get_name()), str(f.full_path))

    def test_cwd(self):
        f = File()
        f.full_path
        f.cwd = 'some/absolute/path'
        self.assertEqual(os.path.join('some', 'absolute', 'path', f.get_name()), str(f.full_path))

class TestPipeFile(unittest.TestCase):

    def test_empty_name(self):
        f = PipeFile()
        self.assertEqual('[base].[pipe].[extension]', f.get_name())
        self.assertEqual('[base].[pipe].abc', f.get_name(extension='abc'))
        self.assertEqual('[base].[pipe].[extension]', f.get_name(extension=''))
        f.set_name('myfile.data.0.ext')
        self.assertEqual('ext', f.extension)
        self.assertEqual('myfile', f.base)
        self.assertEqual('.data.0', f.pipe)
        self.assertEqual('0', f.version)
        self.assertEqual(f.get_name(), str(f.path))

    def test_set_property(self):
        pf = PipeFile()
        self.assertEqual('[base].[pipe].[extension]', pf.get_name())
        with self.assertRaises(NameError):
            pf.base = 'awesome'
        pf.set_name('hello_world.1.png')
        pf.base = 'awesome'
        self.assertEqual('awesome', pf.nice_name)
        self.assertEqual({'base': 'awesome', 'version': '1', 'extension': 'png'}, pf.values)

        p = PipeFile('my_pipe_file.1.png')
        self.assertEqual({'base': 'my_pipe_file', 'version': '1', 'extension': 'png'}, p.values)
        p.extension = 'abc'
        self.assertEqual('my_pipe_file.1.abc', p.name)
        p.output = 'geometry'
        self.assertEqual('my_pipe_file.geometry.1.abc', p.name)
        p.version = 17
        self.assertEqual('my_pipe_file.geometry.17.abc', p.name)
        p.output = None
        self.assertEqual('my_pipe_file.geometry.17.abc', p.name)
        p.version = 0
        self.assertEqual('my_pipe_file.geometry.0.abc', p.name)
        self.assertEqual('PipeFile("my_pipe_file.geometry.0.abc")', repr(p))


class TestDrops(unittest.TestCase):

    def test_empty_name(self):
        Dropper = type('Dropper', (PipeFile,), dict(config=dict(without=r'[a-zA-Z0-9]+', basename=r'[a-zA-Z0-9]+'),
                                                    drops=('base',)))
        d = Dropper()
        self.assertEqual('[without]_[basename].[pipe].[extension]', d.get_name())
        self.assertEqual('awesome_[basename].[pipe].[extension]', d.get_name(without='awesome'))
        self.assertEqual('[without]_replaced.[output].[version].101.[extension]',
                         d.get_name(basename='replaced', frame=101))

        Subdropper = type('Dropper', (Dropper,), dict(config=dict(subdrop='[\w]')))
        s = Subdropper()
        self.assertEqual('[without]_[basename]_[subdrop].[pipe].[extension]', s.get_name())
        self.assertEqual('awesome_[basename]_[subdrop].[pipe].[extension]', s.get_name(without='awesome'))
        self.assertEqual('[without]_replaced_[subdrop].[output].[version].101.[extension]',
                         s.get_name(basename='replaced', frame=101))


class TestCompound(unittest.TestCase):

    def test_empty_name(self):
        Compound = type('Compound', (PipeFile,), dict(config=dict(first=r'[\d]+', second=r'[a-zA-Z]+'),
                                                      compounds=dict(base=('first', 'second'))))
        c = Compound()
        self.assertEqual('[base].[pipe].[extension]', c.get_name())
        self.assertEqual('[base].[pipe].[extension]', c.get_name(first=50))
        self.assertEqual('50abc.[pipe].[extension]', c.get_name(first=50, second='abc'))
        c.set_name(c.get_name(base='101dalmatians', version=1, extension='png'))
        self.assertEqual('101dalmatians', c.nice_name)
        self.assertEqual(
            {'base': '101dalmatians', 'first': '101', 'second': 'dalmatians', 'version': '1', 'extension': 'png'},
            c.values)
        self.assertEqual('200dalmatians.1.png', c.get_name(first=200))


class TestPropertyField(unittest.TestCase):

    def test_empty_name(self):
        class PropertyField(PipeFile):
            config = dict(extrafield='[a-z0-9]+')

            @property
            def nameprop(self):
                return 'staticvalue'

            @property
            def pathprop(self):
                return 'propertyfield'

            def get_path_pattern_list(self):
                result = super().get_pattern_list()
                result.append('pathprop')
                return result

            def get_pattern_list(self):
                result = super().get_pattern_list()
                result.append('nameprop')
                return result

        pf = PropertyField()
        self.assertEqual(Path('[base]/[extrafield]/[pathprop]/[base]_[extrafield]_[nameprop].[pipe].[extension]'), pf.path)
        pf.set_name('simple_property_staticvalue.1.abc')
        self.assertEqual({'base': 'simple', 'extrafield': 'property', 'version': '1', 'extension': 'abc'},
                         pf.values)
        self.assertEqual('simple_property_staticvalue', pf.nice_name)
        self.assertEqual(Path('simple/property/propertyfield/simple_property_staticvalue.1.abc'), pf.path)



#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
    name='naming',
    version='0.1.2.1',
    packages=find_packages(),
    description='Object-oriented names for the digital era.',
    author='Christian López Barrón',
    author_email='chris.gfz@gmail.com',
    url='https://github.com/chrizzFTD/naming',
    download_url='https://github.com/chrizzFTD/naming/releases/tag/0.1.2',
    classifiers=['Programming Language :: Python :: 3.6'],
    extras_require={'docs': ['sphinx_autodoc_typehints']}
)
